@@ -1,64 +1,62 @@
-'''
-Clase Individual, solo es llamada por la clase poblacion y contiene
-informacion sobre un individuo concreto
-esto es:
-str sexo: male or female
-str ide: identificador que consta de generacion (g.X) + numero (ID-YYY)
-int age: de momento sin usar
-dict genotype: {'str': 'str'} siendo la clave A o B y los valores
-                su genotipo para ese locus
-'''
 from calendar import c
-import itertools
 import numpy as np
-from random import randint
 import random
+from random import randint
+
+#local import
 from .functions import outer_product
 
 class Individual():
 
 
-    def __init__(self,nom='1',name='n',
-                 size=1,ploidy=2,
-                 vida_media=0,
+    def __init__(self,nom='1',ploidy=2,
                  freq={'A':(0.4,0.6),'B':(0.6,0.4)},
                  d=0,R=0.5,mu=(0.1,0.1),
                  sex_system='XY',gen=0,
                  parents=0):
+        '''
+        Creates an individual based on population characteristics.
         
-         
-        self.spName = name
-        self.spSize = size
+        Parameters:
+            size (int): population size
+            ploidy (int): number of homologous chromosomes. Defaults to 2.
+            d (float): Linkage desequilibrium. Defaults to 0.
+            R (float): recombination frequency. Defaults to 0.5.
+            mu (tuple): mutation rate. Defaults to (0.1,0.1).
+            sex_system (str): Sex system. Defaults to 'XY'.
+            gen (int): current generation. Defaults to 0.
+            parents (tuple,int): individual parents. Defaults to 0 (no parents).
+            
+        '''
+        
+        #population attributes
         self.spPloidy = ploidy
-
-        self.vida_media = vida_media
+        self.d = d
+        self.R = R
+        self.mu = mu
+        self.alFreq = freq
         
+        #individual attributes
         self.ide = 'g'+str(gen)+".ID-"+str(nom)
         self.age = 0
         self.parents = parents
         self.sex_system = sex_system
-        self.d = d
-        # diccionario con frecuencias alelicas
-        self.alFreq = freq
-        # frecuencia de recombinacion
-        self.R = R
-        # tasa de mutacion
-        self.mu = mu
+        
         self.__createIndividual()
         
 
         
     def __createIndividual(self):
         '''
-        Inicializa las variables que no se le pasan al inicializador
+        Initialize variables that are not passed to the initializer
         '''
         self.chromosome = dict()
-        self.isMutated = False
+        self.isMutated = 0
         # si tiene padres, es decir, no esta iniciada de 0...
         if self.parents:
             # self.mating()
             self.mating()
-            self.mutation()
+            self.isMutated = self.mutation()
         else:
             self.sex_chromosome = self.generate_sexual_genotype()
             self.chromosome = self.chooseGametes()
@@ -69,11 +67,11 @@ class Individual():
 
             
     def generate_sexual_genotype(self):
-        '''Genera un genotipo sexual, compuesto por dos valores correspondientes
-        a cada un de los cromosomas, los almacena en la variable sex_chromosome
+        '''Generates a sexual genotype, composed of two corresponding values
+        to each of the chromosomes, stores them in the variable sex_chromosome
         
-        Returns:
-            str: cromosoma sexual para el individuo'''
+        returns:
+            str: sex chromosome for the individual'''
         # sex_chromosome= 'XX'or'XY' / 'ZW' or 'ZZ' / 'XO' or 'XX'
         if randint(0,1)==0:
             return self.sex_system[0]+self.sex_system[0]
@@ -81,10 +79,11 @@ class Individual():
             return self.sex_system
         
     def getSex(self):
-        '''Devuelve el sexo segun los chromosomas sexuales
+        '''Returns sex for given sexual chromosome. 
         
         Returns:
-            str: 'Male' or 'Female' '''
+            str: sex (Male/Female)
+            '''
             
         if self.sex_chromosome[0] == self.sex_chromosome[1]:
             sexo = 'Female' if self.sex_system != 'ZW' else 'Male'
@@ -93,17 +92,17 @@ class Individual():
         return sexo
         
     def getGenotype(self):
+        '''Obtain the genotype of the individual
+        Returns:
+            dict(str:str): genotype of the individual'''
         
-        genotype = dict()
-        # print(self.chromosome)
-        for i,letra in enumerate(self.chromosome['c1']):
-            if self.spPloidy == 2:
-                genotype[letra.upper()] = ''.join(sorted(self.chromosome['c1'][i] + 
-                                                         self.chromosome['c2'][i]))
-            else:
-                genotype[letra.upper()] = self.chromosome['c1'][i]
+        c = [list(self.chromosome[x]) for x in self.chromosome]
+        array = np.array(c)
+        genotype = [''.join(array[:,i]) for i in range(array.shape[1])]
         
-        return genotype
+        gen_sorted = [''.join(sorted(g)) for g in genotype]
+        
+        return gen_sorted
 
     # Calcula la frecuencia gametica a partir de las frecuencias alelicas y D
     # esto quiza deberia estar en population !!!revisar
@@ -154,8 +153,13 @@ class Individual():
     
     # metodo dunder
     def __str__(self):
-        return ("Este individuo es {}, su sexo es {} su genotipo es {}"
-              .format(self.ide,self.sex,self.chromosome))
+        ind_info = {'Individual id:': self.ide,
+                    'sex': self.sex, 
+                    'genotype': ' '.join(self.genotype)}
+        if self.parents != 0:
+            ind_info['suffered mutation']= False if not self.isMutated else self.isMutated
+            
+        return ('\n'.join(f'{key}: {value}' for key,value in ind_info.items()))
     
     def printParents(self,):
         '''
@@ -168,6 +172,8 @@ class Individual():
 
     # calculates
     def mating(self):
+        ''' Calculates the mating of the individual: parent recombination,
+        sexual chromosomes and gametes.'''
         # print(self.chromosome,len(self.chromosome))
         if self.spPloidy > 1:
             r = self.R
@@ -176,7 +182,7 @@ class Individual():
                 c1P = self.parents[x].chromosome['c1']
                 c2P = self.parents[x].chromosome['c2']
                 # metodo de recombinacion
-                c1,c2 = self.recombination(c1P,c2P,r)              
+                c1,c2 = Individual.recombination(c1P,c2P,r)              
                 self.chromosome['c'+str(x+1)] = random.choice([c1,c2])   
             # cromosomas sexuales sex_chromosome: str XX/XY or ZW/ZZ or XX/X0
             sP = self.parents[0].sex_chromosome
@@ -185,42 +191,46 @@ class Individual():
         else:  
             self.chromosome = {'c'+str(k+1):v for k in range(2) for v in random.choice(self.parents[k].chromosome.values())}
 
-    def recombination(self,c1,c2,r):
-        # factor de interferencia
-        # para que sea menos probable la recombinacion de dos loci seguidos
+    def recombination(c1,c2,r):
+        '''Recombination of two chromosomes by r value.
+        If r=0, the chromosomes are not recombined.'''
+            # factor de interferencia
+            # para que sea menos probable la recombinacion de dos loci seguidos
         int = 0
-        for i in range(0,len(c1),2):
+        c1 = list(c1)
+        c2 = list(c2)
+        c1R,c2R = c1.copy(),c2.copy()
+        for i in range(0,len(c1)):
             if random.random()<=r-int:
                 # ocurre la recombinacion (solo recombinan los loci A B )
-                c1R = c1[i]+c2[i+1:]
-                c2R = c2[i]+c1[i+1:] 
+                c1R = c1
+                c2R = c2
+                c1R[i] = c2[i]
+                c2R[i] = c1[i]
                 int = r/2
                 
-                c1 = c1R
-                c2 = c2R
             else:
-                int = 0
-            
-            
-        return c1,c2
+                int = 0      
+        return ''.join(c1R),''.join(c2R)
             
     
     def mutation(self):
         '''
-        Provoca el cambio del alelo mayor al menor con una frecuencia mut
+        It causes the change from the major to the minor allele with a mut frequency.
         '''
         muType = 'unidirectional'
-        #000100 significa que ha mutado en la posicion 4
-        self.adMutated = ''
-        for k,v in self.chromosome.items():
-            for i in range(len(v)):
-                #comprueba si es el alelo mayor
-                if v[i].isupper():
-                    #si muta, cambia el alelo del cromosoma por el alelo menor
-                    if random.random() < self.mu[i]:
-                        self.chromosome[k] = self.chromosome[k].replace(v[i],v[i].lower())
-                        self.isMutated = True
-                        self.adMutated += k
+        muLoci = list()
+        if sum(self.mu) != 0:      
+            for k,v in self.chromosome.items():
+                for i in range(len(v)):
+                    #comprueba si es el alelo mayor
+                    if v[i].isupper():
+                        #si muta, cambia el alelo del cromosoma por el alelo menor
+                        if random.random() < self.mu[i]:
+                            self.chromosome[k] = self.chromosome[k].replace(v[i],v[i].lower())
+                            muLoci.append(v[i])
+        
+        return muLoci
                     
 
             
