@@ -1,11 +1,8 @@
-'''
-Clase Poblacion, permite crear una nueva poblacion, hacerla evolucionar
-y obtener resumen de sus caracteristicas.
-'''
 
-from cProfile import label
+#local imports
 from .individual import Individual
 from .functions import fitness,outer_product
+from .plot import Plot
 
 import random
 import itertools
@@ -13,6 +10,10 @@ import pandas as pd
 import matplotlib.pyplot as plt
 import numpy as np 
 import re
+import os
+from IPython.display import clear_output
+from matplotlib.ticker import (MultipleLocator, FormatStrFormatter,
+                               AutoMinorLocator)
 
 
 
@@ -21,10 +22,10 @@ import re
 class Population:
 
 
-    def __init__(self,size = 100,name="Population",ploidy = 2, vida_media=55,
-                 R=0.5,mu = (1e-4,1e-4),freq={'A':(0.5,0.5),'B':(0.5,0.5)},D=0,
+    def __init__(self,size = 100,name="Population",ploidy = 2,
+                 R=0.5,mu = (0,0),freq={'A':(0.5,0.5),'B':(0.5,0.5)},D=0,
                  fit=0,sex_system='XY',rnd=False):
-        """Creates a new empty object population.
+        """Creates a new empty population object.
         
         Parameters:
             size (int): Population size. Defaults to 100.
@@ -46,18 +47,16 @@ class Population:
         self.name = name
         self.size = size
         self.ploidy = ploidy
-        self.vida_media = vida_media
         self.d = D
         self.R = R
-        self.steps = 0
+        self.steps = 1
         
         self.rnd = rnd
         
-        #frecuencia genotipica inicial
-        self.freq = self.allelicFreq(freq)
-        self.mu = mu
+        #frecuencia alelica inicial
+        self.freq = self.initialAlleles(freq)
         self.gen = 0
-        self.mu = mu
+        self.generationList = [0]
         
         # stops evolve if needed
         self.stopEv = False
@@ -66,9 +65,10 @@ class Population:
         
         self.sex_system = sex_system.upper()
         
+        self.mu = self.lenMutFreq(mu)
         self.__checkRandom()
         
-    def allelicFreq(self,freq):
+    def initialAlleles(self,freq):
         """
         Checks if dictionary has the correct format
 
@@ -91,6 +91,15 @@ class Population:
                 freq[k]=(v,1-v)
         return freq
     
+    def lenMutFreq(self,mu):
+        freqSize = len(self.freq)
+        muList = list(mu)
+        while(freqSize > len(muList)): 
+            muList.append(0)
+        
+        return tuple(muList)
+        
+    
     def __checkRandom(self):
         '''
         Check if user set random (rnd) to True, then changes R, D, 
@@ -102,7 +111,7 @@ class Population:
             if rnd:
                 self.R = random.random()/2
                 self.d = random.random()/2
-                self.mu = tuple([random.random()/2 for x in range(2)])
+                self.mu = tuple([random.random()/2 for x in range(len(self.freq))])
                 self.fit = random.randint(0,3)
                 
     def __str__(self):
@@ -118,11 +127,8 @@ class Population:
             that as a generation 0 population. Defaults to 0.
         '''
         if not pop:     
-            self.indiv = [Individual(i,
-                                    self.name,
-                                    self.size,
+            self.individuals = [Individual(i,
                                     self.ploidy,
-                                    self.vida_media,
                                     self.freq,
                                     self.d,
                                     self.R,
@@ -131,28 +137,39 @@ class Population:
                                     self.gen,
                                     parents=0) 
                         for i in range(self.size)]
-            print("se han generado un total de {} individuos de la poblacion"
-                .format(self.size))
-        else:
+            print(f"Se han generado {self.size} individuos en base a los atributos de la población")
+        elif isinstance(pop,Population):
             # la poblacion sera una parte o toda, dependiendo del tamaño original
-            self.indiv = pop.indiv[0:self.size if self.size < len(pop.indiv) else len(pop.indiv)]
-            print(f"Se han tomado {self.size} individuos de la poblacion")
+            # ERROR: si se quiere crear una población de una población
+            self.individuals = Population.getCurrentIndividuals(pop)
+            print(f"La población se ha iniciado con {len(self.individuals)} individuos de la población introducida ")
+        elif isinstance(pop,list) and isinstance(pop[0],Individual):
+            self.individuals = pop
+            print(f"La población se ha iniciado con {len(self.individuals)} individuos introducidos ")
+        else:
+            print(f"{type(pop)} no es válido")
+            
         
         # se crean nuevas variables de la poblacion
         # print(self.freq,self.alleleFreq())
-        freq_alelicas = self.alleleFreq()
+        self.obs_initial_all_freq= self.alleleFreq()
         # frecuencia alelica acumulada = se añadiran valores durante la ev
-        self.f_ale_acc = {k: [v] for k,v in freq_alelicas.items()}
+        self.f_ale_acc = {k: [v] for k,v in self.obs_initial_all_freq.items()}
         dictc = self.gameticFreq()
         # frecuencia gametica acumulada
         self.f_gam_acc = {k: [v] for k,v in dictc.items()}
         # frecuencia de mutacion acumulada
-        self.f_mut_acc = [self.findMutated()]
+        self.f_mut_acc = {k: [v] for k,v in self.findMutated().items()}
         # frecuencia de sexos acumulada
-        self.f_sex_acc = [self.sexFreq()]
+        self.f_sex_acc = {k: [v] for k,v in self.sexFreq().items()}
         
-           
-    def printIndiv(self,show=5,children=True):
+    def getCurrentSize(self):
+        '''Returns current population size based on number of individuals in the population
+        
+        '''
+        return len(self.individuals)   
+       
+    def printIndividuals(self,show=5,children=True):
         '''
         Shows information about the individuals. 
         
@@ -169,7 +186,7 @@ class Population:
             print("print chidren")
             objectList = self.childrenInd
         else:
-            objectList = self.indiv
+            objectList = self.individuals
             
         for x in objectList:
             print (*[getattr(x,y) for y in listaAtrib],sep="\t")
@@ -183,63 +200,41 @@ class Population:
         Graphical representation with matplotlib.
         Allelic and gametic frequencies, sex frequency and number of mutations.
         '''  
-
-        # creamos el indice (eje x) del dataFrame
- 
-        labels = ['gen.'+str(x) for x in range(0,self.gen+1,self.steps)]
-        
-        # DataFrame de frecuencias alelicas acumuladas
-        al_df = pd.DataFrame(self.f_ale_acc,index=labels)
-        # DataFrame de frecuencias gameticas acumuladas
-        gam_df = pd.DataFrame(self.f_gam_acc,index=labels)
-        
-        # DataFrame de sexos
-        sex_df = pd.DataFrame(self.f_sex_acc,index=labels)
+        al_df = self.getDataFrame("alleles")
+        gam_df = self.getDataFrame("gametes")
+        sex_df = self.getDataFrame("sex")
+        mu_df = self.getDataFrame("mutations")
 
         # Hacemos el grafico
         fig,ax = plt.subplots(2,2,figsize=(13,8))
+        fig.tight_layout(h_pad=2.1)  
         
         plt.style.context("ggplot")
-        plt.suptitle(f"Población con {self.size} individuos",fontsize=18)
-        caption=f"""Caracteristicas iniciales: frecuencia gametica={self.freq}, R={self.R}
-        frecuencia de mutación={self.mu}"""
+        caption=f"""Initial conditions: allelic f.={self.freq}, recombination f.={self.R}
+        mutation f.={self.mu}"""
         plt.figtext(0.5, 0.01, caption, wrap=True, horizontalalignment='center', fontsize=10)
         
-        # fig[0].title('Variacion de las frecuencias gameticas')
         with plt.style.context("ggplot"):
-          ax[0,0].plot(gam_df)
-          ax[0,0].set_title('frecuencias gaméticas')
-          ax[0,0].legend(gam_df.columns)
-          ax[0,0].set_ylim(0,1)
-          ax[0,0].set_ylabel('%')
-          ax[0,0].set_ylim([0,1])
-          # fig[1].title('Variacion de las frecuencias alelicas')
-          ax[0,1].set_title('frecuencias alélicas')
-          ax[0,1].plot(al_df)
-          ax[0,1].legend(al_df.columns)
-          ax[0,1].set_ylim(0,1)
-          ax[0,1].set_ylabel('%')
-          ax[0,1].set_ylim([0,1])
-          
-          ax[1,0].plot(sex_df)
-          ax[1,0].set_title('frecuencia del sexo')
-          ax[1,0].set_ylim(0.3,0.7)
-          ax[1,0].legend(['Female','Male'])
-          ax[1,0].set_ylabel('%')
-          ax[1,0].set_ylim([0,1])
-          
-          ax[1,1].bar(height=self.f_mut_acc[1:],x=labels[1:])
-          ax[1,1].set_title('Número de mutaciones')
-          maxMutLim = max(self.f_mut_acc)
-          ax[1,1].set_ylim(0,maxMutLim+1)
-          ax[1,1].plot(self.f_mut_acc[1:], color='red', linewidth=2)
-          ax[1,1].set_ylabel('individuos mutados')
-        
-        new_steps = int(len(labels)/5) if len(labels)>8 else 1
-        plt.setp(ax, xticks=range(0,len(labels),new_steps), xticklabels=labels[::new_steps])
+            Plot.alleles(al_df,show=False,ax=ax[0,0])
+            Plot.gametes(gam_df,show=False,ax=ax[0,1])
+            Plot.sex(sex_df,show=False,ax=ax[1,0])
+            Plot.mutations(mu_df,show=False,ax=ax[1,1])
+            
+        plt.suptitle(f"Population with {self.size} individuals",fontsize=18)
+        plt.subplots_adjust(top=0.85)
+        plt.tight_layout()
         plt.show()
-
         
+    #to be implemented
+    def createdf(data,index=None,columns=None):
+        '''Create a dataframe from a dictionary'''
+        dataF=pd.DataFrame(data,index=index)
+        if columns is not None:
+            dataF.columns = columns
+        dataF.index.name = 'Generations'
+        
+        return dataF
+    
     def getDataFrame(self,which='mutantes'):
         '''
         Generates a pandas dataframe.
@@ -252,53 +247,71 @@ class Population:
         Returns:
             (pd.DataFrame): Dataframe.
         '''
-        labels = ['gen.'+str(x) for x in range(0,self.gen+1,self.steps)]
+        columns=None
+        labels = [x for x in range(0,self.gen+1,self.steps)]
+        index = self.generationList
         if isinstance(which, str):
             if re.match('(.+?)?gamet(ica|o|e)s?',which):
                 data = self.f_gam_acc
-                Summary=pd.DataFrame(data,index=labels)
+                columns = [f'p({i})' for i in data.keys()]
             elif re.match('(.+?)?all?el(ica|o|e)s?',which):
                 data = self.f_ale_acc
-                Summary = pd.DataFrame(data,index=labels)
+                columns = [f'p({i})' for i in data.keys()]
             elif re.match('(.+?)?sexo?s?',which):
                 data = self.f_sex_acc
-                Summary =pd.DataFrame(data,index=labels,columns=['Female','Male'])
+                columns= [f'f({i})' for i in data.keys()]
             elif re.match('(.+?)?mut(.+?)',which):
                 data = self.f_mut_acc
-                Summary = pd.DataFrame(data,index=labels)
-            else:
-                raise ValueError(f'Unknown {which}')
+                columns = [f'mu({i})' for i in data.keys()]
         elif isinstance(which,list):
             data = which
             Summary = pd.DataFrame(data,index=labels)
         else:
-            raise TypeError(f'Unknown {which}')
-            
+            raise TypeError(f'Unknown {which} dataframe')
         
-
+        Summary = Population.createdf(data,index,columns)
         return Summary
                                   
-    
-    def gameticFreq(self):
-        '''
-        Computes number of different gametes in the population.
+    def gameticCount(self):
+        '''Counts the number of gametes in the population.
+        
+        Returns:
+            dict: dictionary with the number of different gametes in the population.
         '''
         # diccionario tipo {'AB': 0,'Ab':0,...}
         obsGamf = outer_product(self.freq)
         obsGamf = {k:0 for k in obsGamf.keys()}
-        # cuenta las ocurrencias en la poblacion de los distintos genotipos  
-        for individuo in self.indiv:
+        # cuenta las ocurrencias en la poblacion de los distintos genotipos gameticos
+        for individuo in self.individuals:
             for key in obsGamf:
                 if individuo.chromosome['c1'] == key:
                     obsGamf[key] += 1
                 if individuo.chromosome['c2']==key:
                     obsGamf[key] += 1
-                    
-        return {k: v / (2*len(self.indiv)) for k, v in obsGamf.items()}
+        
+        return obsGamf
+    
+    def gameticFreq(self):
+        '''
+        Computes de gametic frequency for this population.
+        
+        Returns:
+            dict: dictionary with the gametic frequencies.
+        '''
+        
+        countGametes = self.gameticCount()
+        
+        freqGametes = {k: v / (2*len(self.individuals)) for k, v in countGametes.items()}     
+           
+        return freqGametes
+    
+    def __setGameticFreq(self,gamf):
+        self.gameticFrequencies = gamf
+    
     
     def alleleFreq(self):
         '''
-        Obtains number of major alleles from gametic frequency for current population.
+        Obtains frequency of major alleles from gametic frequency for current population.
         
         Returns:
             dict (str:int): Keys = locus(A,B,...), Values=Count
@@ -316,6 +329,9 @@ class Population:
             # ...
 
         return obsAleF
+    
+    def __setAllelicFreq(self,alef):
+        self.allelicFrequencies = alef
         
     def sexFreq(self):
         """Obtains sex frequency for current population.
@@ -323,13 +339,13 @@ class Population:
         Returns:
             dict: Keys = Male-Female, Values = frequencies
         """
-        sex=[0,0]
-        for individuo in self.indiv:
+        sex={'Female':0,'Male':0}
+        for individuo in self.individuals:
             if individuo.getSex()=='Female':
-                sex[0]+=1
+                sex['Female']+=1
             else:
-                sex[1]+=1
-        return [i/len(self.indiv) for i in sex]
+                sex['Male']+=1
+        return {k:(v/len(self.individuals)) for k,v in sex.items()}
     
             
     
@@ -339,7 +355,7 @@ class Population:
         '''
 
         obsGamf = self.gameticFreq()
-
+        self.__setGameticFreq(obsGamf)
         # print(f'Generacion {self.gen}','frecuencia absoluta: ',obsGamf,sep='\n')
         # print(self.cum_gamF)
 
@@ -352,6 +368,7 @@ class Population:
         Adds current allelic frequency to object variable f_ale_acc
         '''
         obsAleF = self.alleleFreq()
+        self.__setAllelicFreq(obsAleF)
         for k in obsAleF:
             self.f_ale_acc[k].append(obsAleF[k])
         
@@ -359,18 +376,92 @@ class Population:
         '''
         Adds current sex frequency to object variable f_sex_acc
         '''
-        self.f_sex_acc.append(self.sexFreq())
+        sex = self.sexFreq()
+        for k in sex:
+            self.f_sex_acc[k].append(sex[k])
 
     def mutAcumulada(self):
         '''
         Adds current mutation numbers to object variable f_mut_acc
         '''
-        self.f_mut_acc.append(self.findMutated()) 
+        mutations = self.findMutated()
+        for k in mutations:
+            self.f_mut_acc[k].append(mutations[k])
         
-         
-    def evolvePop(self,gens = 50,every=10,ignoreSex=False,printInfo=False):
+    def getObsGenotype(self,locus):
         """
-        Evolves the population 
+        Obtains observed genotypes for current population.
+        
+        Parameters:
+            locus (str): Locus of interest.
+            
+        Returns:
+            dict: Keys = genotype, Values = number of individuals with that genotype.
+        """
+        try:
+            
+            genotype = {f'{locus}{locus}':0,f'{locus}{locus.lower()}':0,f'{locus.lower()}{locus.lower()}':0}
+            
+            for ind in self.individuals:
+                for x in range(0,len(ind.genotype)):
+                    if ind.genotype[x] in genotype.keys():
+                        genotype[ind.genotype[x]] +=1
+                        
+            return genotype
+        except:
+            print(f'Wrong locus, it should be capitalize and one of {self.freq.keys()}')
+        
+    def getExpGenotype(self,locus,what_pop='current'):
+        '''Returns the ected genotype based on observed genotype count .
+        The main use is for testing Hardy-Weinberg equilibrium.
+        
+        Parameters:
+            locus (str): Locus of interest.
+            what_pop (str): What population we are using to "ect" the frequencies. Values are "current",
+            which means that we are using the current population observed genotypes, or "initial",
+            which means that we are using the initial population allele frequencies .
+        
+        Returns:
+            dict: Keys = genotype, Values = number of ected individuals with that genotype.
+        '''
+        
+        if what_pop == 'current':
+            size = self.getCurrentSize()
+            obs = self.getObsGenotype(locus)
+            keys = list(obs.keys())
+            p = (obs[keys[0]]+obs[keys[1]]/2)/size
+            q = (obs[keys[2]]+obs[keys[1]]/2)/size
+        else:
+            size = self.size
+            obs = self.obs_initial_all_freq
+            p = obs[locus]
+            q = 1- p
+            keys = [f'{locus}{locus}',f'{locus}{locus.lower()}',f'{locus.lower()}{locus.lower()}']
+            
+        try:
+            counts = (p**2,2*p*q,q**2)
+            expFreq = dict(zip(keys,counts))
+            
+            expCounts = {k:v*size for k,v in expFreq.items()}
+            
+            return expCounts
+        
+        except:
+            print(f'Wrong locus, it should be capitalize and one of {list(self.freq.keys())}')
+        
+    
+        
+                        
+    def evolvePop(self,gens = 50,every=10,ignoreSex=False,printInfo=False,fit=None):
+        """
+        Evolves the population
+        
+        Args:
+            gens (int): Number of generations to evolve.
+            every (int): Number of generations to print info.
+            ignoreSex (bool): If True, evolution will take as hermaphrodites.
+            printInfo (bool): If True, prints info about the evolution.
+            fit (str): If not None, it will change fitness attribute of the population.
 
         Parameters:
             gens (int, optional): Number of generations. Defaults to 50.
@@ -381,7 +472,8 @@ class Population:
         """
         
         self.steps = every
-            
+        if fit is not None:
+            self.fit = fit
         for veces in range(0,gens):
             # si hay que parar la evolucion por algun motivo, sale del bucle
             if self.stopEv:
@@ -390,13 +482,13 @@ class Population:
             #aumentamos la generacion
             self.gen += 1
             #hacemos que poblacion apunte a la lista padre
-            currentPop = self.indiv
+            currentPop = self.individuals
             #vaciamos la lista individuos
             self.childrenInd = []
 
             # introduce nuevos individuos hasta llegar al size de la poblacion
             x = 0
-            while len(self.childrenInd)<= self.size:
+            while len(self.childrenInd) < self.size:
                 child = self.__chooseMate(x, currentPop, ignoreSex)
                 # aplicamos una funcion fitness
                 if fitness(self.fit,child.genotype) == True:
@@ -405,7 +497,7 @@ class Population:
 
             #sobreescribimos la generacion padre por la hija
             if self.stopEv == False:
-                self.indiv = self.childrenInd
+                self.individuals = self.childrenInd
 
             # cada x generaciones, printamos
             if self.gen % every == 0:
@@ -422,8 +514,13 @@ class Population:
                 
                 completed = ((veces+1)/gens)*100
                 if completed < 100:
+                    os.system('cls')
+                    clear_output(wait=True)
                     print(f"{round(completed,1)}% completado...")
         else:
+            clear_output(wait=True)
+            os.system('cls')
+            # self.getInfo()
             print("¡Evolucion completada!")
                 
         
@@ -460,10 +557,7 @@ class Population:
         Ind_Name = x
         # genera un nuevo individuo y lo devuelve al metodo evolvePop
         return Individual(Ind_Name,
-                         self.name,
-                         self.size,
                          self.ploidy,
-                         self.vida_media,
                          self.freq,
                          self.d,
                          self.R,
@@ -480,28 +574,7 @@ class Population:
         self.freqAleAcumulada()
         self.sexAcumulada()
         self.mutAcumulada()
-
-    def printSummary(self):
-        """
-        Get summary about current population.
-        
-        Will be obsolete.
-        """
-        tam = len(self.indiv)
-
-        sex = {'Male':0,'Female':0}
-        for x in range(tam):
-            sexo = self.indiv[x].sex()
-            if sexo == 'Male':
-                sex['Male'] = sex['Male'] + 1
-            else:
-                sex['Female'] =sex['Female']+ 1
-
-        print(f'Hay {len(self.indiv)} individuos\n{sex} son machos\t',
-                f'{sex} son hembras \n\n el desequilibrio de ligamiento (LD) =',
-                f'{self.d} \n frecuencia de recombinacion = {self.R} ',
-                f' la generacion es {self.gen} las frecuencias gameticas', 
-                f'hasta esta generacion son {self.f_gam_acc}')
+        self.generationList.append(self.gen)
 
     def printParentIndividuals(self,id=0):
         """Shows an individual an its parents
@@ -509,37 +582,97 @@ class Population:
         Parameters:
             id (int, optional): which individual will be shown. Defaults to 0.
         """
-        print(self.indiv[id])
-        self.indiv[id].printParents()
+        print(self.individuals[id])
+        self.individuals[id].printParents()
 
     
     def findMutated(self,show=0):
         """Find mutated individuals
 
-        Args:
+        Parameters:
             show (int, optional): If this search will be shown.
             Defaults to 0.
 
         Returns:
-            int: number of individuals which were mutated.
+            dict: number of individuals which were mutated per loci.
         """
         mutated = 0
-        advMutated = {k:0 for k in self.freq.values()}
-        for individuo in self.indiv:
-            if individuo.isMutated:    
+        advMutated = {k:0 for k in self.freq.keys()}
+        for individuo in self.individuals:
+            muValue = individuo.isMutated
+            if muValue:    
                 mutated += 1
+                for x in muValue:
+                    advMutated[x] += 1
                 if show > mutated:
                     print("¡Un individuo ha mutado!, ha ocurrido",
                           f"en la generación {self.gen}",
                           " y se trata de:")
                     print(individuo)
-            # if individuo.adMutated:
-            #     for i in individuo.adMutated:
-            #         advMutated[i] += 1
                     
-        return mutated
+        return advMutated
 
+    def info(pop):
+        info = {'tamaño':pop.size,
+                'ploidía':pop.ploidy, 
+                'frecuencias alelicas iniciales':pop.freq, 
+                'desequilibrio de ligamiento':pop.d, 
+                'frecuencia de recombinacion':pop.R, 
+                'tasa de mutaciones':pop.mu, 
+                'generación actual':pop.gen,
+                'sistema de determinación del sexo': pop.sex_system,
+                'tipo de selección': pop.fit}
+        if hasattr(pop,'indiv'):
+            info['frecuencias alélicas actuales'] = pop.alleleFreq()
+            info['número de individuos'] = len(pop.individuals)
+        
+        stringInfo = '\n'.join([f'{key}: {value}' for key, value in info.items()])
+        print(stringInfo)
+        
+        
+    def getCurrentIndividuals(self,howMany=None,shuffle=False):
+        '''Returns a list of current individuals
+        
+        Parameters:
+            howMany (int, optional): how many individuals will be returned.
+            shuffle (bool, optional): if the individuals will be shuffled.
+        
+        Returns:
+            list: list of current individuals
+        '''
+        individuos = self.individuals.copy()
+        
+        if shuffle:
+            random.shuffle(individuos)
+ 
+        if howMany is not None and howMany < self.size:
+            individuos = individuos[:howMany]
 
+        return individuos
+    
+    def fixedLoci(self):
+        '''Returns which loci, if any, are fixed
+        
+        Returns:
+            list: list of fixed loci
+        '''
+        loci = list()
+        for k,v in self.allelicFrequencies.items():
+            if v==0:
+                loci.append(k.lower())
+            elif v==1:
+                loci.append(k)
+            else:
+                loci.append(0)
+            
+        
+        return loci
+    
+    #TODO: implementar esta funcion
+    def __iter__(self):
+        pass
+    def __next__(self):
+        pass
 
    
 if __name__ == '__main__':
